@@ -48,10 +48,11 @@ extern GtkBuilder *main_builder;
 
 GtkWidget *expressiontext;
 GtkTextBuffer *expressionbuffer;
-GtkTextTag *expression_par_tag;
+GtkTextTag *expression_par_tag, *expression_par_u_tag;
 int expression_lines = -1;
 size_t undo_index = 0;
 int block_add_to_undo = 0;
+int block_emodified = 0;
 vector<string> expression_history;
 string current_history_expression;
 int expression_history_index = -1;
@@ -64,6 +65,8 @@ bool cursor_has_moved = false;
 extern bool minimal_mode;
 bool use_custom_expression_font = false, save_custom_expression_font = false;
 string custom_expression_font;
+int replace_expression = KEEP_EXPRESSION;
+bool highlight_background = false;
 
 string sdot, saltdot, sdiv, sslash, stimes, sminus;
 
@@ -92,6 +95,10 @@ bool read_expression_edit_settings_line(string &svar, string &svalue, int &v) {
 	} else if(svar == "custom_expression_font") {
 		custom_expression_font = svalue;
 		save_custom_expression_font = true;
+	} else if(svar == "replace_expression") {
+		replace_expression = v;
+	} else if(svar == "highlight_parenthesis_background") {
+		highlight_background = v;
 	} else if(!read_expression_completion_settings_line(svar, svalue, v)) {
 		return false;
 	}
@@ -101,6 +108,8 @@ void write_expression_edit_settings(FILE *file) {
 	if(expression_lines > 0) fprintf(file, "expression_lines=%i\n", expression_lines);
 	fprintf(file, "use_custom_expression_font=%i\n", use_custom_expression_font);
 	if(use_custom_expression_font || save_custom_expression_font) fprintf(file, "custom_expression_font=%s\n", custom_expression_font.c_str());
+	fprintf(file, "replace_expression=%i\n", replace_expression);
+	if(highlight_background) fprintf(file, "highlight_parenthesis_background=%i\n", highlight_background);
 	write_expression_completion_settings(file);
 }
 bool read_expression_history_line(string &svar, string &svalue) {
@@ -400,19 +409,13 @@ void focus_keeping_selection() {
 void focus_expression() {
 	if(expression_edit_widget() && !gtk_widget_is_focus(expression_edit_widget())) gtk_widget_grab_focus(expression_edit_widget());
 }
-GtkTextIter istart_save, iend_save;
-void expression_save_selection() {
-	gtk_text_buffer_get_selection_bounds(expression_edit_buffer(), &istart_save, &iend_save);
-}
-void expression_restore_selection() {
-	gtk_text_buffer_select_range(expression_edit_buffer(), &istart_save, &iend_save);
-}
 void expression_select_all() {
 	GtkTextIter istart, iend;
 	gtk_text_buffer_get_start_iter(expression_edit_buffer(), &istart);
 	gtk_text_buffer_get_end_iter(expression_edit_buffer(), &iend);
 	gtk_text_buffer_select_range(expression_edit_buffer(), &istart, &iend);
 	gtk_text_buffer_remove_tag(expression_edit_buffer(), expression_par_tag, &istart, &iend);
+	gtk_text_buffer_remove_tag(expression_edit_buffer(), expression_par_u_tag, &istart, &iend);
 }
 void brace_wrap() {
 	string expr = get_expression_text();
@@ -505,7 +508,9 @@ void highlight_parentheses() {
 	gtk_text_buffer_get_iter_at_mark(expression_edit_buffer(), &icur, mcur);
 	gtk_text_buffer_get_start_iter(expression_edit_buffer(), &istart);
 	gtk_text_buffer_get_end_iter(expression_edit_buffer(), &iend);
+	bool at_end = gtk_text_iter_equal(&icur, &iend);
 	gtk_text_buffer_remove_tag(expression_edit_buffer(), expression_par_tag, &istart, &iend);
+	gtk_text_buffer_remove_tag(expression_edit_buffer(), expression_par_u_tag, &istart, &iend);
 	bool b = false;
 	b = (gtk_text_iter_get_char(&icur) == ')');
 	if(!b && gtk_text_iter_backward_char(&icur)) {
@@ -530,6 +535,10 @@ void highlight_parentheses() {
 			inext = ipar2;
 			gtk_text_iter_forward_char(&inext);
 			gtk_text_buffer_apply_tag(expression_edit_buffer(), expression_par_tag, &ipar2, &inext);
+		} else {
+			GtkTextIter inext = icur;
+			gtk_text_iter_forward_char(&inext);
+			gtk_text_buffer_apply_tag(expression_edit_buffer(), expression_par_u_tag, &icur, &inext);
 		}
 	} else {
 		b = (gtk_text_iter_get_char(&icur) == '(');
@@ -555,13 +564,55 @@ void highlight_parentheses() {
 				inext = ipar2;
 				gtk_text_iter_forward_char(&inext);
 				gtk_text_buffer_apply_tag(expression_edit_buffer(), expression_par_tag, &ipar2, &inext);
+			} else if(!at_end) {
+				GtkTextIter inext = icur;
+				gtk_text_iter_forward_char(&inext);
+				gtk_text_buffer_apply_tag(expression_edit_buffer(), expression_par_u_tag, &icur, &inext);
 			}
 		}
 	}
 }
 
+guint autocalc_selection_timeout_id = 0;
+gboolean do_autocalc_selection_timeout(gpointer) {
+	autocalc_selection_timeout_id = 0;
+	string expression = get_selected_expression_text();
+	if(expression.length() > 2000 || expression.find_first_not_of(NUMBER_ELEMENTS COMMA SPACES) == string::npos || CALCULATOR->hasToExpression(expression, false, evalops) || contains_plot_or_save(CALCULATOR->unlocalizeExpression(expression, evalops.parse_options))) return FALSE;
+	string str;
+	int had_errors = false, had_warnings = false;
+	CALCULATOR->beginTemporaryStopMessages();
+	string result = CALCULATOR->calculateAndPrint(CALCULATOR->unlocalizeExpression(expression, evalops.parse_options), 100, evalops, printops, AUTOMATIC_FRACTION_OFF, AUTOMATIC_APPROXIMATION_OFF, &str, -1, NULL, true);
+	had_errors = CALCULATOR->endTemporaryStopMessages(NULL, &had_warnings);
+	if(had_errors || result.empty() || result == str || result.find(CALCULATOR->abortedMessage()) != string::npos || result.find(CALCULATOR->timedOutString()) != string::npos) {
+		if(str.empty()) {
+			CALCULATOR->endTemporaryStopMessages();
+			return FALSE;
+		}
+	} else {
+		if(*printops.is_approximate) {
+			if(printops.use_unicode_signs && can_display_unicode_string_function(SIGN_ALMOST_EQUAL, (void*) parse_status_widget())) {
+				str += " " SIGN_ALMOST_EQUAL " ";
+			} else {
+				str += "= ";
+				str += _("approx.");
+				str += " ";
+			}
+		} else {
+			str += " = ";
+		}
+		str += result;
+	}
+	set_status_selection_text(str, had_errors, had_warnings);
+	return FALSE;
+}
+
 extern bool tabbed_completion;
 void on_expressionbuffer_cursor_position_notify() {
+	if(autocalc_selection_timeout_id != 0) {
+		g_source_remove(autocalc_selection_timeout_id);
+		autocalc_selection_timeout_id = 0;
+	}
+	clear_status_selection_text();
 	selstore_end = selstore_start;
 	tabbed_completion = false;
 	cursor_has_moved = true;
@@ -573,9 +624,19 @@ void on_expressionbuffer_cursor_position_notify() {
 	highlight_parentheses();
 	display_parse_status();
 	mainwindow_cursor_moved();
+	GtkTextIter istart, iend;
+	if(auto_calculate && !rpn_mode && !status_blocked() && display_expression_status && gtk_text_buffer_get_selection_bounds(expression_edit_buffer(), &istart, &iend) && (!gtk_text_iter_is_start(&istart) || !gtk_text_iter_is_end(&iend))) {
+		autocalc_selection_timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 700, do_autocalc_selection_timeout, NULL, NULL);
+	}
 }
 
 extern bool tabbed_completion;
+void block_expression_modified() {
+	block_emodified++;
+}
+void unblock_expression_modified() {
+	block_emodified--;
+}
 void set_expression_modified(bool b, bool handle, bool autocalc) {
 	if(!b || !handle) {
 		expression_has_changed = b;
@@ -599,6 +660,7 @@ void set_expression_modified(bool b, bool handle, bool autocalc) {
 }
 
 void on_expressionbuffer_changed(GtkTextBuffer *o, gpointer) {
+	if(block_emodified) return;
 	selstore_end = selstore_start;
 	set_expression_modified(true, true, o != NULL);
 }
@@ -792,57 +854,6 @@ return TRUE;}
 	}
 	if(do_keyboard_shortcut(keyval, state)) return TRUE;
 	FIX_ALT_GR
-	if(rpn_mode && state & GDK_CONTROL_MASK) {
-		switch(keyval) {
-			case GDK_KEY_Up: {
-				stack_view_rotate(true);
-				return TRUE;
-			}
-			case GDK_KEY_Down: {
-				stack_view_rotate(false);
-				return TRUE;
-			}
-			case GDK_KEY_Right: {
-				stack_view_swap();
-				return TRUE;
-			}
-			case GDK_KEY_Left: {
-				stack_view_lastx();
-				return TRUE;
-			}
-			case GDK_KEY_Delete: {}
-			case GDK_KEY_KP_Delete: {
-				if(state & GDK_SHIFT_MASK) {
-					stack_view_clear();
-				} else {
-					stack_view_pop();
-				}
-				return TRUE;
-			}
-			case GDK_KEY_C: {
-				if(state & GDK_SHIFT_MASK) {
-					stack_view_copy();
-					return TRUE;
-				}
-				break;
-			}
-			case GDK_KEY_S: {
-				if(state & GDK_SHIFT_MASK) {
-					stack_view_swap();
-					return TRUE;
-				}
-				break;
-			}
-			case GDK_KEY_L: {
-				if(state & GDK_SHIFT_MASK) {
-					stack_view_lastx();
-					return TRUE;
-				}
-				break;
-			}
-			default: {}
-		}
-	}
 	switch(keyval) {
 		case GDK_KEY_Escape: {
 			if(completion_visible()) {
@@ -1123,11 +1134,6 @@ return TRUE;}
 			overwrite_expression_selection(CALCULATOR->getDecimalPoint().c_str());
 			return TRUE;
 		}
-		/*case GDK_KEY_braceleft: {}
-		case GDK_KEY_braceright: {
-			if(expression_in_quotes()) break;
-			return TRUE;
-		}*/
 	}
 	if(state & GDK_CONTROL_MASK && keyval == GDK_KEY_c && !gtk_text_buffer_get_has_selection(expression_edit_buffer())) {
 		copy_result();
@@ -1550,7 +1556,20 @@ gboolean on_expressiontext_button_press_event(GtkWidget*, GdkEventButton *event,
 	return FALSE;
 }
 
-void update_expression_colors(bool initial, bool text_color_set) {
+void set_expression_highlight_background(bool b) {
+	highlight_background = b;
+	update_expression_colors(false, false);
+}
+bool get_expression_highlight_background() {
+	return highlight_background && get_expression_enable_highlight_background();
+}
+bool get_expression_enable_highlight_background() {
+	GdkRGBA c;
+	gtk_style_context_get_color(gtk_widget_get_style_context(expression_edit_widget()), GTK_STATE_FLAG_NORMAL, &c);
+	return c.green < 0.8;
+}
+
+void update_expression_colors(bool initial, bool keep_pars) {
 #if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 14
 
 	gint scalefactor = gtk_widget_get_scale_factor(GTK_WIDGET(gtk_builder_get_object(main_builder, "expression_button_equals")));
@@ -1588,7 +1607,7 @@ void update_expression_colors(bool initial, bool text_color_set) {
 
 #endif
 
-	if(initial || !text_color_set) {
+	if(initial || !keep_pars) {
 		GdkRGBA c;
 		gtk_style_context_get_color(gtk_widget_get_style_context(expression_edit_widget()), GTK_STATE_FLAG_NORMAL, &c);
 #if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION < 16
@@ -1599,25 +1618,60 @@ void update_expression_colors(bool initial, bool text_color_set) {
 		}
 #endif
 		GdkRGBA c_par = c;
-		if(c_par.green >= 0.8) {
-			c_par.red /= 1.5;
-			c_par.blue /= 1.5;
+		if(c.green >= 0.8) {
+			c_par.red /= 2;
+			c_par.blue /= 2;
 			c_par.green = 1.0;
 		} else {
-			if(c_par.green >= 0.5) c_par.green = 1.0;
-			else c_par.green += 0.5;
+			if(highlight_background) {
+				c_par.red = 0.7;
+				c_par.green = 0.95;
+				c_par.blue = 0.75;
+			} else {
+				if(c_par.green >= 0.5) c_par.green = 1.0;
+				else if(c_par.green > 0.1) c_par.green += 0.5;
+				else c_par.green = 0.6;
+			}
+		}
+		GdkRGBA c_par_u = c;
+		if(c.green >= 0.8) {
+			c_par_u.red = 1.0;
+			c_par_u.blue /= 2;
+			c_par_u.green /= 2;
+		} else {
+			if(highlight_background) {
+				c_par_u.red = 1;
+				c_par_u.green = 0.8;
+				c_par_u.blue = 0.8;
+			} else {
+				if(c_par_u.red >= 0.5) c_par_u.red = 1.0;
+				else if(c_par_u.red > 0.2) c_par_u.red += 0.5;
+				else c_par_u.red = 0.7;
+			}
 		}
 		if(initial) {
-			PangoLayout *layout_par = gtk_widget_create_pango_layout(expression_edit_widget(), "()");
+			PangoLayout *layout_par = gtk_widget_create_pango_layout(expression_edit_widget(), "(1)");
 			gint w1 = 0, w2 = 0;
 			pango_layout_get_pixel_size(layout_par, &w1, NULL);
-			pango_layout_set_markup(layout_par, "<b>()</b>", -1);
+			pango_layout_set_markup(layout_par, "<b>(</b>1<b>)</b>", -1);
 			pango_layout_get_pixel_size(layout_par, &w2, NULL);
 			g_object_unref(layout_par);
-			if(w1 == w2) expression_par_tag = gtk_text_buffer_create_tag(expression_edit_buffer(), "curpar", "foreground-rgba", &c_par, "weight", PANGO_WEIGHT_BOLD, NULL);
-			else expression_par_tag = gtk_text_buffer_create_tag(expression_edit_buffer(), "curpar", "foreground-rgba", &c_par, NULL);
+			if(c.green >= 0.8 || !highlight_background) {
+				if(w1 == w2) expression_par_tag = gtk_text_buffer_create_tag(expression_edit_buffer(), "curpar", "foreground-rgba", &c_par, "weight", PANGO_WEIGHT_BOLD, NULL);
+				else expression_par_tag = gtk_text_buffer_create_tag(expression_edit_buffer(), "curpar", "foreground-rgba", &c_par, NULL);
+				if(w1 == w2) expression_par_u_tag = gtk_text_buffer_create_tag(expression_edit_buffer(), "curpar_u", "foreground-rgba", &c_par_u, "weight", PANGO_WEIGHT_BOLD, NULL);
+				else expression_par_u_tag = gtk_text_buffer_create_tag(expression_edit_buffer(), "curpar_u", "foreground-rgba", &c_par_u, NULL);
+			} else {
+				if(w1 == w2) expression_par_tag = gtk_text_buffer_create_tag(expression_edit_buffer(), "curpar", "background-rgba", &c_par, "weight", PANGO_WEIGHT_BOLD, NULL);
+				else expression_par_tag = gtk_text_buffer_create_tag(expression_edit_buffer(), "curpar", "background-rgba", &c_par, NULL);
+				if(w1 == w2) expression_par_u_tag = gtk_text_buffer_create_tag(expression_edit_buffer(), "curpar_u", "background-rgba", &c_par_u, "weight", PANGO_WEIGHT_BOLD, NULL);
+				else expression_par_u_tag = gtk_text_buffer_create_tag(expression_edit_buffer(), "curpar_u", "background-rgba", &c_par_u, NULL);
+			}
 		} else {
-			g_object_set(G_OBJECT(expression_par_tag), "foreground-rgba", &c_par, NULL);
+			if(c.green >= 0.8 || !highlight_background) g_object_set(G_OBJECT(expression_par_tag), "foreground-rgba", &c_par, "background-rgba", NULL, NULL);
+			else g_object_set(G_OBJECT(expression_par_tag), "background-rgba", &c_par, "foreground-rgba", NULL, NULL);
+			if(c.green >= 0.8 || !highlight_background) g_object_set(G_OBJECT(expression_par_u_tag), "foreground-rgba", &c_par_u, "background-rgba", NULL, NULL);
+			else g_object_set(G_OBJECT(expression_par_u_tag), "background-rgba", &c_par_u, "foreground-rgba", NULL, NULL);
 		}
 	}
 }
@@ -1653,13 +1707,15 @@ void expression_font_modified() {
 	while(gtk_events_pending()) gtk_main_iteration();
 	set_expression_size_request();
 	set_expression_operator_symbols();
-	PangoLayout *layout_par = gtk_widget_create_pango_layout(expression_edit_widget(), "()");
+	PangoLayout *layout_par = gtk_widget_create_pango_layout(expression_edit_widget(), "(1)");
 	gint w1 = 0, w2 = 0;
 	pango_layout_get_pixel_size(layout_par, &w1, NULL);
-	pango_layout_set_markup(layout_par, "<b>()</b>", -1);
+	pango_layout_set_markup(layout_par, "<b>(</b>1<b>)</b>", -1);
 	pango_layout_get_pixel_size(layout_par, &w2, NULL);
 	if(w1 == w2) g_object_set(expression_par_tag, "weight", PANGO_WEIGHT_BOLD, NULL);
 	else g_object_set(expression_par_tag, "weight", PANGO_WEIGHT_NORMAL, NULL);
+	if(w1 == w2) g_object_set(expression_par_u_tag, "weight", PANGO_WEIGHT_BOLD, NULL);
+	else g_object_set(expression_par_u_tag, "weight", PANGO_WEIGHT_NORMAL, NULL);
 }
 
 void update_expression_font(bool initial) {
