@@ -122,7 +122,7 @@ void create_application(GtkApplication *app) {
 	//create the almighty Calculator object
 	new Calculator(ignore_locale);
 
-	CALCULATOR->setExchangeRatesWarningEnabled(false);
+	CALCULATOR->setExchangeRatesWarningEnabled(!CALCULATOR->canFetch());
 
 	//load application specific preferences
 	load_preferences();
@@ -372,10 +372,11 @@ int main (int argc, char *argv[]) {
 				remove_blank_ends(lang);
 				if(!lang.empty()) {
 #	ifdef _WIN32
-					_putenv_s("LANG", lang.c_str());
+					_putenv_s("LANGUAGE", lang.c_str());
 #	else
 					setenv("LANGUAGE", lang.c_str(), 1);
-					setenv("LC_MESSAGES", lang.c_str(), 1);
+					if(lang.find(".") == string::npos && lang.find("_") != string::npos) lang += ".utf8";
+					if(lang.find(".") != string::npos) setenv("LC_MESSAGES", lang.c_str(), 1);
 #	endif
 				}
 				break;
@@ -391,10 +392,16 @@ int main (int argc, char *argv[]) {
 			if(GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &nlang, NULL, &n)) {
 				WCHAR* wlocale = new WCHAR[n];
 				if(GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &nlang, wlocale, &n)) {
+					for(size_t i = 2; nlang > 1 && i < n - 1; i++) {
+						if(wlocale[i] == '\0') {
+							if(wlocale[i + 1] == '\0') break;
+							wlocale[i] = ':';
+							nlang--;
+						}
+					}
 					lang = utf8_encode(wlocale);
 					gsub("-", "_", lang);
-					if(lang.length() > 5) lang = lang.substr(0, 5);
-					if(!lang.empty()) _putenv_s("LANG", lang.c_str());
+					if(!lang.empty()) _putenv_s("LANGUAGE", lang.c_str());
 				}
 				delete[] wlocale;
 			}
